@@ -2,11 +2,12 @@
  * ioctl.c: DVD ioctl replacement function
  *****************************************************************************
  * Copyright (C) 1999-2001 VideoLAN
- * $Id: ioctl.c,v 1.2 2001/06/14 02:47:44 sam Exp $
+ * $Id: ioctl.c,v 1.11 2001/08/10 17:43:50 sam Exp $
  *
  * Authors: Markus Kuespert <ltlBeBoy@beosmail.com>
  *          Samuel Hocevar <sam@zoy.org>
  *          Jon Lech Johansen <jon-vl@nanocrew.net>
+ *          Hkan Hjort <d95hjort@dtek.chalmers.se>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -48,16 +49,28 @@
 #ifdef DVD_STRUCT_IN_LINUX_CDROM_H
 #   include <linux/cdrom.h>
 #endif
+#ifdef DVD_STRUCT_IN_DVD_H
+#   include <dvd.h>
+#endif
+#ifdef DVD_STRUCT_IN_BSDI_DVDIOCTL_DVD_H
+#   include <BSDI_dvdioctl/dvd.h>
+#endif
 #ifdef SYS_BEOS
 #   include <malloc.h>
 #   include <scsi.h>
 #endif
+#ifdef SOLARIS_USCSI
+#   include <unistd.h>
+#   include <stropts.h>
+#   include <sys/scsi/scsi_types.h>
+#   include <sys/scsi/impl/uscsi.h>
+#endif
 
 #include "config.h"
 #include "common.h"
 
-#ifdef SYS_DARWIN1_3
-#   include "DVDioctl/DVDioctl.h"
+#ifdef SYS_DARWIN
+#   include "MacOSX_dvdioctl/DVDioctl.h"
 #endif
 
 #include "ioctl.h"
@@ -69,6 +82,13 @@
 static void BeInitRDC ( raw_device_command *, int );
 #endif
 
+/*****************************************************************************
+ * Local prototypes, Solaris specific
+ *****************************************************************************/
+#if defined( SOLARIS_USCSI )
+static void SolarisInitUSCSI( struct uscsi_cmd *p_sc, int i_type );
+#endif
+
 /*****************************************************************************
  * Local prototypes, win32 (aspi) specific
  *****************************************************************************/
@@ -84,7 +104,7 @@ int ioctl_ReadCopyright( int i_fd, int i_layer, int *pi_copyright )
 {
     int i_ret;
 
-#if defined( DVD_STRUCT_IN_LINUX_CDROM_H )
+#if defined( HAVE_LINUX_DVD_STRUCT )
     dvd_struct dvd;
 
     dvd.type = DVD_STRUCT_COPYRIGHT;
@@ -114,10 +134,22 @@ int ioctl_ReadCopyright( int i_fd, int i_layer, int *pi_copyright )
 
     *pi_copyright = p_buffer[ 4 ];
 
-#elif defined( SYS_DARWIN1_3 )
-    _dvd_error( dvdcss, "DVD ioctls not fully functional yet, "
-                           "assuming disc is encrypted" );
+#elif defined( SOLARIS_USCSI )
+    INIT_USCSI( GPCMD_READ_DVD_STRUCTURE, 8 );
+    
+    rs_cdb.cdb_opaque[ 6 ] = i_layer;
+    rs_cdb.cdb_opaque[ 7 ] = DVD_STRUCT_COPYRIGHT;
 
+    i_ret = ioctl(i_fd, USCSICMD, &sc);
+
+    if( i_ret < 0 || sc.uscsi_status ) {
+        i_ret = -1;
+    }
+    
+    *pi_copyright = p_buffer[ 4 ];
+    // s->copyright.rmi = p_buffer[ 5 ];
+
+#elif defined( SYS_DARWIN )
     *pi_copyright = 1;
 
     i_ret = 0;
@@ -168,6 +200,16 @@ int ioctl_ReadCopyright( int i_fd, int i_layer, int *pi_copyright )
         *pi_copyright = p_buffer[ 4 ];
     }
 
+#elif defined( __QNXNTO__ )
+    /*
+        QNX RTOS currently doesn't have a CAM
+        interface (they're working on it though).
+        Assume DVD is not encrypted.
+    */
+
+    *pi_copyright = 0;
+    i_ret = 0;
+
 #else
     /* DVD ioctls unavailable - do as if the ioctl failed */
     i_ret = -1;
@@ -183,7 +225,7 @@ int ioctl_ReadKey( int i_fd, int *pi_agid, u8 *p_key )
 {
     int i_ret;
 
-#if defined( DVD_STRUCT_IN_LINUX_CDROM_H )
+#if defined( HAVE_LINUX_DVD_STRUCT )
     dvd_struct dvd;
 
     dvd.type = DVD_STRUCT_DISCKEY;
@@ -230,10 +272,23 @@ int ioctl_ReadKey( int i_fd, int *pi_agid, u8 *p_key )
 
     memcpy( p_key, p_buffer + 4, 2048 );
 
-#elif defined( SYS_DARWIN1_3 )
-    _dvd_error( dvdcss, "DVD ioctls not fully functional yet, "
-                           "sending an empty key" );
+#elif defined( SOLARIS_USCSI )
+    INIT_USCSI( GPCMD_READ_DVD_STRUCTURE, 2048 + 4 );
+    
+    rs_cdb.cdb_opaque[ 7 ] = DVD_STRUCT_DISCKEY;
+    rs_cdb.cdb_opaque[ 10 ] = *pi_agid << 6;
+    
+    i_ret = ioctl( i_fd, USCSICMD, &sc );
+    
+    if( i_ret < 0 || sc.uscsi_status )
+    {
+        i_ret = -1;
+        return i_ret;
+    }
 
+    memcpy( p_key, p_buffer + 4, 2048 );
+
+#elif defined( SYS_DARWIN )
     i_ret = 0;
 
     memset( p_key, 0x00, 2048 );
@@ -294,7 +349,7 @@ int ioctl_ReportAgid( int i_fd, int *pi_agid )
 {
     int i_ret;
 
-#if defined( DVD_STRUCT_IN_LINUX_CDROM_H )
+#if defined( HAVE_LINUX_DVD_STRUCT )
     dvd_authinfo auth_info;
 
     auth_info.type = DVD_LU_SEND_AGID;
@@ -323,7 +378,21 @@ int ioctl_ReportAgid( int i_fd, int *pi_agid )
 
     *pi_agid = p_buffer[ 7 ] >> 6;
 
-#elif defined( SYS_DARWIN1_3 )
+#elif defined( SOLARIS_USCSI )
+    INIT_USCSI( GPCMD_REPORT_KEY, 8 );
+    
+    rs_cdb.cdb_opaque[ 10 ] = DVD_REPORT_AGID | (*pi_agid << 6);
+    
+    i_ret = ioctl( i_fd, USCSICMD, &sc );
+    
+    if( i_ret < 0 || sc.uscsi_status )
+    {
+        i_ret = -1;
+    }
+
+    *pi_agid = p_buffer[ 7 ] >> 6;
+    
+#elif defined( SYS_DARWIN )
     INIT_DVDIOCTL( 8 );
 
     dvdioctl.i_keyformat = kCSSAGID;
@@ -371,7 +440,7 @@ int ioctl_ReportChallenge( int i_fd, int *pi_agid, u8 *p_challenge )
 {
     int i_ret;
 
-#if defined( DVD_STRUCT_IN_LINUX_CDROM_H )
+#if defined( HAVE_LINUX_DVD_STRUCT )
     dvd_authinfo auth_info;
 
     auth_info.type = DVD_LU_SEND_CHALLENGE;
@@ -400,7 +469,21 @@ int ioctl_ReportChallenge( int i_fd, int *pi_agid, u8 *p_challenge )
 
     memcpy( p_challenge, p_buffer + 4, 12 );
 
-#elif defined( SYS_DARWIN1_3 )
+#elif defined( SOLARIS_USCSI )
+    INIT_USCSI( GPCMD_REPORT_KEY, 16 );
+    
+    rs_cdb.cdb_opaque[ 10 ] = DVD_REPORT_CHALLENGE | (*pi_agid << 6);
+    
+    i_ret = ioctl( i_fd, USCSICMD, &sc );
+    
+    if( i_ret < 0 || sc.uscsi_status )
+    {
+        i_ret = -1;
+    }
+
+    memcpy( p_challenge, p_buffer + 4, 12 );
+    
+#elif defined( SYS_DARWIN )
     INIT_DVDIOCTL( 16 );
 
     dvdioctl.i_keyformat = kChallengeKey;
@@ -461,7 +544,7 @@ int ioctl_ReportASF( int i_fd, int *pi_agid, int *pi_asf )
 {
     int i_ret;
 
-#if defined( DVD_STRUCT_IN_LINUX_CDROM_H )
+#if defined( HAVE_LINUX_DVD_STRUCT )
     dvd_authinfo auth_info;
 
     auth_info.type = DVD_LU_SEND_ASF;
@@ -492,7 +575,21 @@ int ioctl_ReportASF( int i_fd, int *pi_agid, int *pi_asf )
 
     *pi_asf = p_buffer[ 7 ] & 1;
 
-#elif defined( SYS_DARWIN1_3 )
+#elif defined( SOLARIS_USCSI )
+    INIT_USCSI( GPCMD_REPORT_KEY, 8 );
+    
+    rs_cdb.cdb_opaque[ 10 ] = DVD_REPORT_ASF | (*pi_agid << 6);
+    
+    i_ret = ioctl( i_fd, USCSICMD, &sc );
+    
+    if( i_ret < 0 || sc.uscsi_status )
+    {
+        i_ret = -1;
+    }
+
+    *pi_asf = p_buffer[ 7 ] & 1;
+    
+#elif defined( SYS_DARWIN )
     INIT_DVDIOCTL( 8 );
 
     dvdioctl.i_keyformat = kASF;
@@ -555,7 +652,7 @@ int ioctl_ReportKey1( int i_fd, int *pi_agid, u8 *p_key )
 {
     int i_ret;
 
-#if defined( DVD_STRUCT_IN_LINUX_CDROM_H )
+#if defined( HAVE_LINUX_DVD_STRUCT )
     dvd_authinfo auth_info;
 
     auth_info.type = DVD_LU_SEND_KEY1;
@@ -584,7 +681,21 @@ int ioctl_ReportKey1( int i_fd, int *pi_agid, u8 *p_key )
 
     memcpy( p_key, p_buffer + 4, 8 );
 
-#elif defined( SYS_DARWIN1_3 )
+#elif defined( SOLARIS_USCSI )
+    INIT_USCSI( GPCMD_REPORT_KEY, 12 );
+    
+    rs_cdb.cdb_opaque[ 10 ] = DVD_REPORT_KEY1 | (*pi_agid << 6);
+    
+    i_ret = ioctl( i_fd, USCSICMD, &sc );
+    
+    if( i_ret < 0 || sc.uscsi_status )
+    {
+        i_ret = -1;
+    }
+
+    memcpy( p_key, p_buffer + 4, 8 );;
+    
+#elif defined( SYS_DARWIN )
     INIT_DVDIOCTL( 12 );
 
     dvdioctl.i_keyformat = kKey1;
@@ -639,7 +750,7 @@ int ioctl_InvalidateAgid( int i_fd, int *pi_agid )
 {
     int i_ret;
 
-#if defined( DVD_STRUCT_IN_LINUX_CDROM_H )
+#if defined( HAVE_LINUX_DVD_STRUCT )
     dvd_authinfo auth_info;
 
     auth_info.type = DVD_INVALIDATE_AGID;
@@ -666,7 +777,19 @@ int ioctl_InvalidateAgid( int i_fd, int *pi_agid )
 
     i_ret = ioctl( i_fd, B_RAW_DEVICE_COMMAND, &rdc, sizeof(rdc) );
 
-#elif defined( SYS_DARWIN1_3 )
+#elif defined( SOLARIS_USCSI )
+    INIT_USCSI( GPCMD_REPORT_KEY, 0 );
+    
+    rs_cdb.cdb_opaque[ 10 ] = DVD_INVALIDATE_AGID | (*pi_agid << 6);
+    
+    i_ret = ioctl( i_fd, USCSICMD, &sc );
+    
+    if( i_ret < 0 || sc.uscsi_status )
+    {
+        i_ret = -1;
+    }
+
+#elif defined( SYS_DARWIN )
     INIT_DVDIOCTL( 0 );
 
     dvdioctl.i_keyformat = kInvalidateAGID;
@@ -712,7 +835,7 @@ int ioctl_InvalidateAgid( int i_fd, int *pi_agid )
  *****************************************************************************/
 int ioctl_SendChallenge( int i_fd, int *pi_agid, u8 *p_challenge )
 {
-#if defined( DVD_STRUCT_IN_LINUX_CDROM_H )
+#if defined( HAVE_LINUX_DVD_STRUCT )
     dvd_authinfo auth_info;
 
     auth_info.type = DVD_HOST_SEND_CHALLENGE;
@@ -742,7 +865,22 @@ int ioctl_SendChallenge( int i_fd, int *pi_agid, u8 *p_challenge )
 
     return ioctl( i_fd, B_RAW_DEVICE_COMMAND, &rdc, sizeof(rdc) );
 
-#elif defined( SYS_DARWIN1_3 )
+#elif defined( SOLARIS_USCSI )
+    INIT_USCSI( GPCMD_SEND_KEY, 16 );
+    
+    rs_cdb.cdb_opaque[ 10 ] = DVD_SEND_CHALLENGE | (*pi_agid << 6);
+    
+    p_buffer[ 1 ] = 0xe;
+    memcpy( p_buffer + 4, p_challenge, 12 );
+    
+    if( ioctl( i_fd, USCSICMD, &sc ) < 0 || sc.uscsi_status )
+    {
+        return -1;
+    }
+
+    return 0;
+    
+#elif defined( SYS_DARWIN )
     INIT_DVDIOCTL( 16 );
 
     dvdioctl.i_keyformat = kChallengeKey;
@@ -796,7 +934,7 @@ int ioctl_SendChallenge( int i_fd, int *pi_agid, u8 *p_challenge )
  *****************************************************************************/
 int ioctl_SendKey2( int i_fd, int *pi_agid, u8 *p_key )
 {
-#if defined( DVD_STRUCT_IN_LINUX_CDROM_H )
+#if defined( HAVE_LINUX_DVD_STRUCT )
     dvd_authinfo auth_info;
 
     auth_info.type = DVD_HOST_SEND_KEY2;
@@ -826,6 +964,21 @@ int ioctl_SendKey2( int i_fd, int *pi_agid, u8 *p_key )
 
     return ioctl( i_fd, B_RAW_DEVICE_COMMAND, &rdc, sizeof(rdc) );
 
+#elif defined( SOLARIS_USCSI )
+    INIT_USCSI( GPCMD_SEND_KEY, 12 );
+    
+    rs_cdb.cdb_opaque[ 10 ] = DVD_SEND_KEY2 | (*pi_agid << 6);
+    
+    p_buffer[ 1 ] = 0xa;
+    memcpy( p_buffer + 4, p_key, 8 );
+    
+    if( ioctl( i_fd, USCSICMD, &sc ) < 0 || sc.uscsi_status )
+    {
+        return -1;
+    }
+
+    return 0;
+    
 #elif defined( WIN32 )
     if( WIN2K ) /* NT/Win2000/Whistler */
     {
@@ -857,7 +1010,7 @@ int ioctl_SendKey2( int i_fd, int *pi_agid, u8 *p_key )
         return WinSendSSC( i_fd, &ssc );
     }
 
-#elif defined( SYS_DARWIN1_3 )
+#elif defined( SYS_DARWIN )
     INIT_DVDIOCTL( 12 );
 
     dvdioctl.i_keyformat = kKey2;
@@ -913,6 +1066,43 @@ static void BeInitRDC( raw_device_command *p_rdc, int i_type )
 }
 #endif
 
+#if defined( SOLARIS_USCSI )
+/*****************************************************************************
+ * SolarisInitUSCSI: initialize a USCSICMD structure for the Solaris kernel
+ *****************************************************************************
+ * This function initializes a Solaris userspace scsi command structure for 
+ * future use, either a read command or a write command.
+ *****************************************************************************/
+static void SolarisInitUSCSI( struct uscsi_cmd *p_sc, int i_type )
+{   
+    union scsi_cdb *rs_cdb;
+    memset( p_sc->uscsi_cdb, 0, sizeof( union scsi_cdb ) );
+    memset( p_sc->uscsi_bufaddr, 0, p_sc->uscsi_buflen );
+    
+    switch( i_type )
+    {
+        case GPCMD_SEND_KEY:
+	    p_sc->uscsi_flags = USCSI_ISOLATE | USCSI_WRITE;
+            break;
+
+        case GPCMD_READ_DVD_STRUCTURE:
+        case GPCMD_REPORT_KEY:
+	    p_sc->uscsi_flags = USCSI_ISOLATE | USCSI_READ;
+            break;
+    }
+    
+    rs_cdb = (union scsi_cdb *)p_sc->uscsi_cdb;
+    
+    rs_cdb->scc_cmd = i_type;
+
+    rs_cdb->cdb_opaque[ 8 ] = (p_sc->uscsi_buflen >> 8) & 0xff;
+    rs_cdb->cdb_opaque[ 9 ] =  p_sc->uscsi_buflen       & 0xff;
+    p_sc->uscsi_cdblen = 12;
+
+    USCSI_TIMEOUT( p_sc, 15 );
+}
+#endif
+
 #if defined( WIN32 )
 /*****************************************************************************
  * WinInitSSC: initialize a ssc structure for the win32 aspi layer
@@ -975,3 +1165,4 @@ static int WinSendSSC( int i_fd, struct SRB_ExecSCSICmd *p_ssc )
     return p_ssc->SRB_Status == SS_COMP ? 0 : -1;
 }
 #endif
+
